@@ -314,7 +314,7 @@ See also document for canna:saved-exit-minibuffer."
   (canna:finalize)
   )
 
-(add-hook 'kill-emacs 'canna:kill-emacs)
+(add-hook 'kill-emacs-hook 'canna:kill-emacs)
 
 ;;;
 ;;; function for mini-buffer
@@ -475,9 +475,6 @@ kana-to-kanji translation."
   (if (stringp canna-mode-string)
       (mode-line-canna-mode-update canna-mode-string))
 
-  ;; $B8uJdI=<($,$J$1$l$P%U%'%s%9%b!<%I$+$iH4$1$k!#(B
-  (cond (canna-empty-info (canna:quit-canna-mode)))
-
   ;; $B%_%K%P%C%U%!$K=q$/$3$H$,B8:_$9$k$N$G$"$l$P!"$=$l$r%_%K%P%C%U%!(B
   ;; $B$KI=<($9$k!#(B
   (cond (canna-ichiran-string
@@ -490,6 +487,9 @@ kana-to-kanji translation."
 	 (select-frame (window-frame (minibuffer-window)))
 	 (select-window (minibuffer-window))
 	 (use-local-map canna-minibuffer-mode-map) ))
+
+  ;; $B8uJdI=<($,$J$1$l$P%U%'%s%9%b!<%I$+$iH4$1$k!#(B
+  (cond (canna-empty-info (canna:quit-canna-mode)))
   )
 
 (defun canna:minibuffer-input (str len revpos revlen nfixed)
@@ -558,7 +558,7 @@ kana-to-kanji translation."
 	      (canna:insert-fixed nfixed)
 	      (canna:insert-preedit) ))
 
-	(if canna-empty-info
+	(if (and canna-empty-info (> len 0))
 	    (progn
 	      (delete-region (point-min) (point-max))
 	      (message str) ))
