@@ -1056,10 +1056,19 @@ up)
 
 ;; Force `f' rather than `e' in the mode doc:
 (defalias 'dired-advertised-find-file 'dired-find-file)
-(defun dired-find-file ()
-  "In dired, visit the file or directory named on this line."
-  (interactive)
-  (find-file (file-name-sans-versions (dired-get-filename) t)))
+;   >> Shingu, Toshiaki    1993030550  {
+;(defun dired-find-file ()
+;  "In dired, visit the file or directory named on this line."
+;  (interactive)
+;  (find-file (file-name-sans-versions (dired-get-filename) t)))
+;   >> Shingu, Toshiaki    1993030550  }
+;   << Shingu, Toshiaki    1993030550  {
+(defun dired-find-file (&optional arg)
+  "In dired, visit the file or directory named on this line.
+A prefix argument enables user to specify the coding-system interactively."
+  (interactive "ZCoding-system: ")
+  (find-file (file-name-sans-versions (dired-get-filename) t) arg))
+;   << Shingu, Toshiaki    1993030550  }
 
 (defun dired-mouse-find-file-other-window (event)
   "In dired, visit the file or directory name you click on."
@@ -1084,10 +1093,19 @@ otherwise, display it in another buffer."
 	  (dired (dired-get-filename)))
     (view-file (dired-get-filename))))
 
-(defun dired-find-file-other-window ()
-  "In dired, visit this file or directory in another window."
-  (interactive)
-  (find-file-other-window (file-name-sans-versions (dired-get-filename) t)))
+;   >> Shingu, Toshiaki    1993030550  {
+;(defun dired-find-file-other-window ()
+;  "In dired, visit this file or directory in another window."
+;  (interactive)
+;  (find-file-other-window (file-name-sans-versions (dired-get-filename) t)))
+;   >> Shingu, Toshiaki    1993030550  }
+;   << Shingu, Toshiaki    1993030550  {
+(defun dired-find-file-other-window (&optional arg)
+  "In dired, visit this file or directory in another window.
+A prefix argument enables user to specify the coding-system interactively."
+  (interactive "ZCoding-system: ")
+  (find-file-other-window (file-name-sans-versions (dired-get-filename) t) arg))
+;   << Shingu, Toshiaki    1993030550  }
 
 (defun dired-display-file ()
   "In dired, display this file or directory in another window."
