@@ -588,6 +588,11 @@
 (define-key menu-mode-map "\C-l" 'menu:refresh)
 ;;; 0 .. 9 a .. z A .. z
 (define-key menu-mode-map "\C-m" 'menu:select)
+(define-key menu-mode-map [return] 'menu:select)
+(define-key menu-mode-map [left] 'menu:previous-item)
+(define-key menu-mode-map [right] 'menu:next-item)
+(define-key menu-mode-map [up] 'menu:previous-item-old)
+(define-key menu-mode-map [down] 'menu:next-item-old)
 
 ;; 92.6.14 by T.Enami -- This function was completely modified.
 (defun menu:select-from-menu (menu &optional initial position)
@@ -656,24 +661,24 @@
 	 )
 	;; end of patch
 	(while (not finished)
-	  (let ((ch (read-char-exclusive)))
+	  (let ((ch (read-event)))
 	    (setq quit-flag nil)
 	    (cond
-	     ((= ch ?\C-a)
+	     ((eq ch ?\C-a)
 	      (menu:select-goto-item 0))
-	     ((= ch ?\C-e)
+	     ((eq ch ?\C-e)
 	      (menu:select-goto-item (1- (length menu:*select-items*))))
-	     ((= ch ?\C-f)
+	     ((or (eq ch ?\C-f) (eq ch 'right))
 	      ;;(menu:select-goto-item (1+ menu:*select-item-no*))
 	      (menu:select-next-item)
 	      )
-	     ((= ch ?\C-b)
+	     ((or (eq ch ?\C-b) (eq ch 'left))
 	      ;;(menu:select-goto-item (1- menu:*select-item-no*))
 	      (menu:select-previous-item)
 	      )
-	     ((= ch ?\C-n)
+	     ((or (eq ch ?\C-n) (eq ch 'down))
 	      (menu:select-goto-menu (1+ menu:*select-menu-no*)))
-	     ((= ch ?\C-g)
+	     ((eq ch ?\C-g)
 	      (if menu:*select-menu-stack*
 		  (let ((save (car menu:*select-menu-stack*)))
 		    (setq menu:*select-menu-stack*
@@ -693,20 +698,20 @@
 		    )
 		(setq finished t
 		      value nil)))
-	     ((= ch ?\C-p)
+	     ((or (eq ch ?\C-p) (eq ch 'up))
 	      (menu:select-goto-menu (1- menu:*select-menu-no*)))
-	     ((= ch ?\C-l)  ;;; redraw menu
+	     ((eq ch ?\C-l)  ;;; redraw menu
 	      (menu:select-goto-menu menu:*select-menu-no*))
-	     ((and (<= ?0 ch) (<= ch ?9)
+	     ((and (numberp ch) (<= ?0 ch) (<= ch ?9)
 		   (<= ch (+ ?0 (1- (length menu:*select-items*)))))
 	      (menu:select-goto-item (- ch ?0)))
-	     ((and (<= ?a ch) (<= ch ?z)
+	     ((and (numberp ch) (<= ?a ch) (<= ch ?z)
 		   (<= (+ 10 ch) (+ ?a (1- (length menu:*select-items*)))))
 	      (menu:select-goto-item (+ 10 (- ch ?a))))
-	     ((and (<= ?A ch) (<= ch ?Z) ; patch by enami@ptgd.sony.co.jp
+	     ((and (numberp ch) (<= ?A ch) (<= ch ?Z)
 		   (<= (+ 10 ch) (+ ?A (1- (length menu:*select-items*)))))
 	      (menu:select-goto-item (+ 10 (- ch ?A))))
-	     ((= ch ?\C-m)
+	     ((or (eq ch ?\C-m) (eq ch 'return))
 	      (setq value (menu:item-value (nth menu:*select-item-no* 
 						menu:*select-items*)))
 	      (setq menu:*select-positions* 
@@ -1618,7 +1623,7 @@ OUTPUTn)))$B$O(B INPUT $B$,F~NO$5$l$?;~$K>uBV(B STATE $B$rI=<($7!$>r7o(B C
 (defun its:peek-char ()
   (if (= (point) its:*buff-e*)
       (if its:*interactive*
-	  (setq unread-command-events (list (read-char-exclusive)))
+	  (setq unread-command-events (list (read-event)))
 	nil)
     (following-char)))
 
@@ -1627,7 +1632,7 @@ OUTPUTn)))$B$O(B INPUT $B$,F~NO$5$l$?;~$K>uBV(B STATE $B$rI=<($7!$>r7o(B C
       (progn 
 	(setq its:*char-from-buff* nil)
 	(if its:*interactive*
-	    (read-char-exclusive)
+	    (read-event)
 	  nil))
     (let ((ch (following-char)))
       (setq its:*char-from-buff* t)
@@ -1816,18 +1821,18 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
   (let ((point (point))
 	(output (cdr action-output))
 	(ch 0))
-    (while (not (= ch ?\^L))
+    (while (not (eq ch ?\^L))
       (insert "<" (nth (car action-output)output) ">")
-      (setq ch (read-char-exclusive))
-      (cond ((= ch ?\^N)
+      (setq ch (read-event))
+      (cond ((eq ch ?\^N)
 	     (setcar action-output
 		     (mod (1+ (car action-output)) (length output))))
-	    ((= ch ?\^P)
+	    ((eq ch ?\^P)
 	     (setcar action-output
 		     (if (= 0 (car action-output))
 			 (1- (length output))
 		       (1- (car action-output)))))
-	    ((= ch ?\^M)
+	    ((eq ch ?\^M)
 	     (setcar (nthcdr 2 its:*select-alternative-output-menu* )
 		     output)
 	     (let ((values 
@@ -1837,7 +1842,7 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
 	       (cond((consp values)
 		     (setcar action-output (nth 1 values))
 		     (setq ch ?\^L)))))
-	    ((= ch ?\^L)
+	    ((eq ch ?\^L)
 	     )
 	    (t
 	     (beep)
@@ -2161,13 +2166,14 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
 (defvar minibuffer-window-selected nil)
 
 (defun egg:select-window-hook (old new)
-  (cond((eq old (minibuffer-window))
-	(save-excursion
-	  (set-buffer (window-buffer (minibuffer-window)))
-	  (setq minibuffer-preprompt nil
-		egg:*mode-on* (default-value 'egg:*mode-on*)
-		egg:*input-mode* (default-value 'egg:*input-mode*)
-		egg:*in-fence-mode* (default-value 'egg:*in-fence-mode*)))))
+  (if (and (eq old (minibuffer-window))
+           (not (eq new (minibuffer-window))))
+      (save-excursion
+	(set-buffer (window-buffer (minibuffer-window)))
+	(setq minibuffer-preprompt nil
+	      egg:*mode-on* (default-value 'egg:*mode-on*)
+	      egg:*input-mode* (default-value 'egg:*input-mode*)
+	      egg:*in-fence-mode* (default-value 'egg:*in-fence-mode*))))
   (if (eq new (minibuffer-window))
       (setq minibuffer-window-selected t)
     (setq minibuffer-window-selected nil)))
