@@ -19,7 +19,7 @@
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;; 94.10.13 created for Mule Ver.2.1 by TAKAHASHI Naoto <ntakahas@etl.go.jp>
-;;; 94.12.16 modified by TAKAHASHI Naoto <ntakahas@etl.go.jp>
+;;; 94.12.27 modified for Mule Ver.2.2 by TAKAHASHI Naoto <ntakahas@etl.go.jp>
 
 ;;
 ;; ETHIOPIC UTILITY FUNCTIONS
@@ -326,7 +326,7 @@ buffer is read-only."
 	    (insert ?$(2$Q(B))
 
 	   ; soft ascii mode, pre = \, fol = ;
-	   ((= fol 59)
+	   ((= fol ?\;)
 	    (delete-char 1)
 	    (insert ?$(2$R(B))
 
@@ -376,7 +376,7 @@ buffer is read-only."
 	  (setq ascii-mode t))
 
 	 ; soft ethio mode, pre = \, fol = {. , ; : | ' `}
-	 ((memq fol '(?. ?, 59 ?: ?| ?' ?`))
+	 ((memq fol '(?. ?, ?\; ?: ?| ?' ?`))
 	  (forward-char 1))
 
 	 ; soft ethio mode, pre = \, fol = others
@@ -603,31 +603,32 @@ in Ethio start mode.  The markers will not be removed."
    "Pe" "Pu" "Pi" "Pa" "PE" "P" "Po"
    "pe" "pu" "pi" "pa" "pE" "p" "po"
    " " "\\_" "." "," ";" ":" "\\*" "\\<" "\\>"
-   "\\1" "\\2" "\\3" "\\4" "\\5" "\\6" "\\7" "\\8" "\\9"
-   "\\10" "\\20" "\\30" "\\40" "\\50" "\\60" "\\70" "\\80" "\\90"
-   "\\100" "\\10000"
+   "1" "2" "3" "4" "5" "6" "7" "8" "9"
+   "10" "20" "30" "40" "50" "60" "70" "80" "90"
+   "100" "10000"
    "\\~e" "\\~E" "\\~a" "\\~A"])
 
 (defvar ethio-use-tigrigna-style nil
-  "If non-NIL, use \"e\" instead of \"a\" for the first lone vowel translation
-in sera-to-fidel and fidel-to-sera conversions.")
+  "*If non-NIL, use \"e\" instead of \"a\" for the first lone vowel
+translation in sera-to-fidel and fidel-to-sera conversions.")
 
 (defvar ethio-quote-vowel-always nil
-  "If non-NIL, lone vowels are always transcribed by
-\"an apostrophe + the vowel\" except at word initial.
-Otherwise, they are quoted by an apostrophe only if the preceding
-Ethiopic character is a lone consonant.")
+  "*If non-NIL, lone vowels are always transcribed by \"an apostrophe
++ the vowel\" except at word initial.  Otherwise, they are quoted by
+an apostrophe only if the preceding Ethiopic character is a lone
+consonant.")
 
 (defvar ethio-W-sixth-always nil
-  "If non-NIL, the Wu-form of a 12-form consonant is transcribed by \"W'\"
-instead of \"Wu\".")
+  "*If non-NIL, the Wu-form of a 12-form consonant is transcribed by
+\"W'\" instead of \"Wu\".")
 
 (defvar ethio-numeric-reduction 0
-  "Degree of reduction in transcribing Ethiopic digits by Arabic digits.
-For example, $(2$`$_$i$g$](B ({10}{9}{100}{80}{7}) will be transcribed by:
-    \10\9\100\80\7		if ETHIO-NUMERIC-REDUCTION is 0,
-    \109100807						   is 1,
-    \10900807						   is 2.")
+  "*Degree of reduction in transcribing Ethiopic digits by Arabic
+digits.  For example, $(2$`$_$i$g$](B ({10}{9}{100}{80}{7}) will be
+transcribed by:
+    \10\9\100\80\7	if ETHIO-NUMERIC-REDUCTION is 0,
+    \109100807					   is 1,
+    \10900807					   is 2.")
 
 ;;;###autoload
 (defun fidel-to-sera-region (begin end &optional ascii-mode force)
@@ -684,95 +685,92 @@ ethio-quote-vowel-on-demand and ethio-numeric-reduction."
     (goto-char (point-min))
     (while (not (eobp))
       (setq ch (following-char))
-      (cond
 
-       ((= ch ?\\ )
-	(insert "\\")
-	(forward-char 1)
-	(setq lonec nil
-	      fidel nil
-	      digit nil))
+      ;; ethiopic charactes
+      (if (= (char-leading-char ch) lc-ethio)
+	  (progn
+	    (setq ch (char-to-ethiocode ch))
+	    (delete-char 1)
 
-       ((memq ch '(?$(2$O(B ?$(2$T(B ?$(2$U(B ?$(2$V(B ?$(2$k(B ?$(2$l(B ?$(2$m(B ?$(2$n(B))
-	(delete-char 1)
-	(insert (aref fidel-to-sera-map (char-to-ethiocode ch)))
-	(setq lonec nil
-	      fidel nil
-	      digit nil))
+	    (cond
 
-       ((memq ch '(?$(2$P(B ?$(2$Q(B ?$(2$R(B ?$(2$S(B))
-	(delete-char 1)
-	(if ascii-mode
-	    (insert "\\"))
-	(insert (aref fidel-to-sera-map (char-to-ethiocode ch)))
-	(setq lonec nil
-	      fidel nil
-	      digit nil))
-
-       ((memq ch '(?. ?, 59 ?: ?| ?' ?`)) ; 59 = semicolon
-	(if (not ascii-mode)
-	    (insert "\\"))
-	(forward-char 1)
-	(setq lonec nil
-	      fidel nil
-	      digit nil))
-
-       ((= (char-leading-char ch) lc-ethio)
-	(setq ch (char-to-ethiocode ch))
-	(delete-char 1)
-	(if ascii-mode
-	    (progn
-	      (insert "\\ ")
+	     ;; fidels
+	     ((<= ch 326)
+	      (if ascii-mode
+		  (insert "\\ "))
+	      (if (and (memq ch '(144 145 146 147 148 150 151)) ;;  (auiAEoe3)
+		       (or lonec
+			   (and ethio-quote-vowel-always
+				fidel)))
+		  (insert "'"))
+	      (insert (aref fidel-to-sera-map ch))
 	      (setq ascii-mode nil
-		    lonec nil
+		    lonec (ethio-lone-consonant-p ch)
+		    fidel t
+		    digit nil))
+
+	     ;; punctuations and symbols
+	     ((or (< ch 336) (> ch 355))
+	      (if (and ascii-mode
+		       (memq ch '(329 330 331 332))) ;;  (.,;:)
+		  (insert "\\"))
+	      (insert (aref fidel-to-sera-map ch))
+	      (setq lonec nil
 		    fidel nil
-		    digit nil)))
-	(if (and (memq ch '(144 145 146 147 148 150 151)) ; lone vowel - `I'
-		 (or lonec
-		     (and ethio-quote-vowel-always
-			  fidel)))
-	    (insert "'"))
-	(setq lonec (ethio-lone-consonant-p ch)
-	      fidel (<= ch 326))	; 326 = po (last fidel in charset)
+		    digit nil))
+
+	     ;; now CH must be an ethiopic digit
+
+	     ;; reduction = 0 or leading digit
+	     ((or (= ethio-numeric-reduction 0)
+		  (not digit))
+	      (insert "\\" (aref fidel-to-sera-map ch))
+	      (setq lonec nil
+		    fidel nil
+		    digit t))
+
+	     ;; reduction = 2 and following 10s, 100s, 10000s 
+	     ((and (= ethio-numeric-reduction 2)
+		   (memq ch '(345 354 355)))
+	      (insert (substring (aref fidel-to-sera-map ch) 1))
+	      (setq lonec nil
+		    fidel nil
+		    digit t))
+
+	     ;; ordinary following digits
+	     (t
+	      (insert (aref fidel-to-sera-map ch))
+	      (setq lonec nil
+		    fidel nil
+		    digit t))))
+
+	;; non-ethiopic characters
 	(cond
 
-	 ; not an Ethiopic digit
-	 ((not (ethio-digit-p ch))
-	  (insert (aref fidel-to-sera-map ch))
-	  (setq digit nil))
+	 ;; backslash is always quoted
+	 ((= ch ?\\ )
+	  (insert "\\"))
 
-	 ; the first of continuous digits, or no reduction
-	 ((or (not digit)
-	      (= ethio-numeric-reduction 0))
-	  (insert (aref fidel-to-sera-map ch))
-	  (setq digit t))
+	 ;; nothing to do if in ascii-mode
+	 (ascii-mode)
 
-	 ; the second or later in continuous digits, reduction = 1
-	 ((= ethio-numeric-reduction 1)
-	  (insert (substring (aref fidel-to-sera-map ch) 1)))
+	 ;; ethio-mode -> ascii-mode
+	 ((or (and (>= ch ?a) (<= ch ?z))
+	      (and (>= ch ?A) (<= ch ?Z))
+	      (memq ch '(?| ?' ?`)))
+	  (insert "\\ ")
+	  (setq ascii-mode t))
 
-	 ; reduction = 2 and (ch = 10 or 100 or 10000)
-	 ((or (= ch 345) (= ch 354) (= ch 355))
-	  (insert (substring (aref fidel-to-sera-map ch) 2)))
+	 ;; ascii punctuations in ethio-mode
+	 ((memq ch '(?. ?, ?\; ?:))
+	  (insert "\\")))
 
-	 (t
-	  (insert (substring (aref fidel-to-sera-map ch) 1)))))
-
-       ((or (and (>= ch ?A) (<= ch ?Z))
-	    (and (>= ch ?a) (<= ch ?z)))
-	(if (not ascii-mode)
-	    (progn
-	      (insert "\\ ")
-	      (setq ascii-mode t)))
-	(forward-char 1))
-
-       (t
 	(forward-char 1)
 	(setq lonec nil
 	      fidel nil
-	      digit nil))))
+	      digit nil)))
 
-    ; a few modifications for readability
+    ;; a few modifications for readability
     (goto-char (point-min))
     (while (re-search-forward "\\([]!\"#$%&()*+/<=>?@[^_-]+\\)\\\\ " nil t)
       (replace-match "\\\\ \\1"))
@@ -795,10 +793,6 @@ ethio-quote-vowel-on-demand and ethio-numeric-reduction."
    ((< code 327)
     (= (mod code 7) 3))))
 
-(defun ethio-digit-p (code)
-  "If the ethiocode CODE is an Ethiopic digit, return t."
-  (and (>= code 336) (<= code 355)))
-
 ;;;###autoload
 (defun fidel-to-sera-mail nil
   "Does FIDEL to SERA conversion for reading/writing mail and news.
