@@ -119,6 +119,11 @@
     (modify-syntax-entry (make-character lc-cn c) "w")
     (setq c (1+ c)))
 
+  (modify-syntax-entry (make-character lc-cns1) "w")
+  (modify-syntax-entry (make-character lc-cns2) "w")
+  (modify-syntax-entry (make-character lc-big5-1) "w")
+  (modify-syntax-entry (make-character lc-big5-2) "w")
+
   ;; For syntax of Latin-1 characters.
   (setq c 192)				; from ',A@(B' to ',A(B'
   (while (< c 256)
