@@ -422,7 +422,7 @@ machine readable simple format."
       (insert "\t##############################\n"
 	      "\t### DIAGNOSIS OF YOUR MULE ###\n"
 	      "\t##############################\n\n"
-	      "CONTENTS: Section 0.  General infomation\n"
+	      "CONTENTS: Section 0.  General information\n"
 	      "          Section 1.  Display\n"
 	      "          Section 2.  Input methods\n"
 	      "          Section 3.  Coding-systems\n"
@@ -431,7 +431,7 @@ machine readable simple format."
 	  (insert "          Section 5.  Fontset list\n"))
       (insert "\n")
 
-      (insert-section 0 "General infomation")
+      (insert-section 0 "General information")
       (insert "Mule's version: " mule-version " of " mule-version-date "\n")
       (if window-system
 	  (insert "Window-system: "
