@@ -41,6 +41,7 @@
   (setq purify-flag nil)
   (load "mule.el")
   (load "mule-conf.el")
+  (load "help")
   (load "simple.el")
   (load "files.el")
   (garbage-collect)
@@ -50,6 +51,7 @@
   (load "paragraphs" nil)
   (load "lisp-mode" nil)
   (load "bytecomp.el" nil)
+  (load "mule-util.el" nil)
   (garbage-collect)
   )
 
