@@ -363,6 +363,53 @@ A prefix argument enables user to specify the coding-system interactively."
 	(si:find-file-other-window filename))
     (si:find-file-other-window filename)))
 
+(or (fboundp 'si:find-file-read-only-other-window)
+    (fset 'si:find-file-read-only-other-window
+        (symbol-function 'find-file-read-only-other-window)))
+
+(defun find-file-read-only-other-window (filename &optional user-coding-system)
+  "Edit file FILENAME in another window but don't allow changes.\n\
+Like find-file but marks buffer as read-only.\n\
+May create a new window, or reuse an existing one;
+A prefix argument enables user to specify the coding-system interactively."
+  (interactive "FFind file read-only in ohter window: \nZCoding-system: ")
+  (if user-coding-system
+      (let ((file-coding-system-for-read
+           (check-coding-system user-coding-system)))
+      (si:find-file-read-only-other-window filename))
+    (si:find-file-read-only-other-window filename)))
+
+(or (fboundp 'si:find-file-other-frame)
+    (fset 'si:find-file-other-frame
+        (symbol-function 'find-file-other-frame)))
+
+(defun find-file-other-frame (filename &optional user-coding-system)
+  "Edit file FILENAME, in another frame.
+May create a new frame, or reuse an existing one.
+See the function `display buffer'.
+A prefix argument enables user to specify the coding-system interactively."
+  (interactive "FFind file in other frame: \nZCoding-system: ")
+  (if user-coding-system
+      (let ((file-coding-system-for-read
+           (check-coding-system user-coding-system)))
+      (si:find-file-other-frame filename))
+    (si:find-file-other-frame filename)))
+
+(or (fboundp 'si:find-file-read-only-other-frame)
+    (fset 'si:find-file-read-only-other-frame
+        (symbol-function 'find-file-read-only-other-frame)))
+
+(defun find-file-read-only-other-frame (filename &optional user-coding-system)
+  "Edit file FILENAME in another frame but don't allow changes.
+Like find-file-ohter-frame but marks buffer as read-only.
+A prefix argument enables user to specify the coding-system interactively."
+  (interactive "FFind file read-only in other frame: \nZCoding-system: ")
+  (if user-coding-system
+      (let ((file-coding-system-for-read
+           (check-coding-system user-coding-system)))
+      (si:find-file-read-only-other-frame filename))
+    (si:find-file-read-only-other-frame filename)))
+
 (or (fboundp 'si:insert-file)
     (fset 'si:insert-file (symbol-function 'insert-file)))
 
@@ -574,8 +621,7 @@ Please redefine this function as you wish."
 	  
 (setq start-process-hook 'default-start-process-hook
       open-network-stream-hook 'default-open-network-stream-hook
-      call-process-hook 'default-call-process-hook
-      call-process-region-hook 'default-call-process-region-hook)
+      call-process-hook 'default-call-process-hook)
 
 ;;;
 ;;;  program --> coding-system translation
