@@ -1,11 +1,12 @@
-;; Basic multilingual commands for Mule
-;; Copyright (C) 1992 Free Software Foundation, Inc.
+;; mule.el --- basic multilingual commands for Mule
+;; Copyright (C) 1992,93,94,95 Free Software Foundation, Inc.
+
 ;; This file is part of Mule (MULtilingual Enhancement of GNU Emacs).
 
 ;; Mule is free software distributed in the form of patches to GNU Emacs.
 ;; You can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 1, or (at your option)
+;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
 ;; Mule is distributed in the hope that it will be useful,
@@ -17,6 +18,8 @@
 ;; along with GNU Emacs; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
+;;; Commentary:
+
 ;;; 87.6.15  created by K.Handa
 ;;; 87.6.24  modified by K.Handa & H.Nakahara<a32275@tansei.u-tokyo.junet>
 ;;; 88.1.18  modified for Nemacs Ver.2.0 by K.Handa
@@ -172,12 +175,12 @@
 ;;; 94.4.6   modified for Mule Ver.1.1 by K.Handa <handa@etl.go.jp>
 ;;;	Improve post-read-conversion/pre-write-conversion handling.
 
-(provide 'mule)				;93.6.25 by T.Enami
+;;; Code:
 
-(defconst mule-version "2.2 (WAKAMURASAKI)" "\
+(defconst mule-version "2.2 (WAKAMURASAKI) PL02" "\
 Version number and name of this version of Mule.")
 
-(defconst mule-version-date "1994.11.28" "\
+(defconst mule-version-date "1995.2.17" "\
 Distribution date of this version of Mule.")
 
 (defun mule-version () "\
@@ -195,7 +198,7 @@ Return string describing the version of Mule that is running."
 ;; 93.7.23 by K.Handa
 ;; Alternative key definitions
 ;; Original mapping will be altered by set-keyboard-coding-system.
-(define-key esc-map "#" 'spell-word)	;originally "$"
+(define-key esc-map "#" 'ispell-word)	;originally "$"
 (define-key esc-map "{" 'insert-parentheses) ;originally "("
 
 (defvar mc-verbose-code nil
@@ -422,6 +425,9 @@ is equal to:
 	(list lc-cn ?c "Chinese 2-byte character.")
 	(list lc-jp ?j "Japanese 2-byte character.")
 	(list lc-kr ?h "Hungul 2-byte character.")
+	(list lc-jp2 ?j "Japanese 2-byte character.")
+	(list lc-cns1 ?t "Chinese(taiwan) 2-byte character.")
+	(list lc-cns2 ?t "Chinese(taiwan) 2-byte character.")
 	(list lc-big5-1 ?t "Big5 Level 1.")
 	(list lc-big5-2 ?t "Big5 Level 2.")
 	)
@@ -442,7 +448,7 @@ Each element is a list of leading-character, mnemonic, and description")
 
 ;;; At the present, I know Japanese and Chinese text can
 ;;; break line at any point under a ristriction of 'kinsoku'.
-(defvar word-across-newline "\\(\\cj\\|\\cc\\)"
+(defvar word-across-newline "\\(\\cj\\|\\cc\\|\\ct\\)"
   "Regular expression of such characters which can be a word across newline.")
 
 ;; Coding-sytem staffs
@@ -803,7 +809,7 @@ of the symbols *euc-japan*, ...) used for reading the file or a list.
 If the return value is a list, insert-file-contents assumes that the
 function has inserted the file for itself and supresses further
 reading and just returns this list.  The elements of list should be
-abosolute pathname and length of data inserted.")
+absolute pathname and length of data inserted.")
 
 (defvar insert-file-contents-error-hook nil
   "A hook function to set file-coding-system of the current buffer.
@@ -812,7 +818,7 @@ On file-error while reading, insert-file-contents calls it with arguments
  FILENAME, VISIT [same as those given to insert-file-contents],
  and a cons (SIGNALED-CONDITIONS . SIGNAL-DATA).
 Usually this is the case of the file not existing.
-The error condition propagets to the caller of insert-file-contents.")
+The error condition propagates to the caller of insert-file-contents.")
 
 ;; 93.7.8 by N.Demizu
 (defvar insert-file-contents-post-hook nil
@@ -821,7 +827,7 @@ The error condition propagets to the caller of insert-file-contents.")
 After successful reading, insert-file-contents calls it with arguments
  FILENAME, VISIT [same as those given to insert-file-contents],
  and RETURN-VALUE of si:insert-file-contents, which is a list of:
- coding-system used for reading, absoule pathname, length of data inserted.")
+ coding-system used for reading, absolute pathname, length of data inserted.")
 
 ;;94.3.22 by K.Handa
 (defvar insert-file-contents-set-coding-system-function
@@ -898,7 +904,7 @@ it means replace the current buffer contents (in the accessible portion)
 with the file contents.  This is better than simply deleting and inserting
 the whole thing because (1) it preserves some marker positions
 and (2) it puts less data in the undo list.
-Code convserion occurs accoding to the value of file-coding-system-for-read.
+Code conversion occurs according to the value of file-coding-system-for-read.
 If current buffer's file-coding-system is nil,
  it is set to the coding-system which is actually used for reading.
 See also insert-file-contents-pre-hook, insert-file-contents-error-hook,
@@ -958,7 +964,7 @@ Before writing, write-region calls it with arguments
  given to write-region].
 The return value of this function should be a coding-system or a list.
  If list, write-region supresses further writing.  The elements of list
- should be a return value of write-region (i.e. list of an abosolute pathname
+ should be a return value of write-region (i.e. list of an absolute pathname
  and length of data written).")
 
 (defvar write-region-post-hook nil
@@ -1196,3 +1202,6 @@ See also open-network-stream-hook and si:open-network-stream."
 	(si:open-network-stream name buf host service
 				(car coding-systems) (cdr coding-systems)))))
 
+(provide 'mule)
+
+;;; mule.el ends here
