@@ -88,7 +88,10 @@
 (require 'faces)
 
 ;;;###autoload
-(defconst quail-version "2.1")
+(defconst quail-version "2.2")
+
+(defvar quail-region-face 'underline)
+(defvar quail-selection-face 'highlight)
 
 (defvar quail-self-insert-after-hook nil
   "Function(s) to call after inserting quail characters.")
@@ -564,7 +567,7 @@ The description about the current quail package is shown by \\[quail-help]."
   (if (null (overlayp quail-overlay))
       (progn
 	(setq quail-overlay (make-overlay (point) (point)))
-	(overlay-put quail-overlay 'face 'underline)))
+	(overlay-put quail-overlay 'face quail-region-face)))
   (make-local-variable 'post-command-hook)
   (add-hook 'post-command-hook 'quail-reset-state)
   (setq quail-keep-state nil)
@@ -930,7 +933,7 @@ The description about the current quail package is shown by \\[quail-help]."
 	      (if (= num (1+ (% idx 10)))
 		  (setq p1 p p2 (point)))
 	      (setq candidates (cdr candidates)))
-	    (put-text-property p1 p2 'face 'region))))
+	    (put-text-property p1 p2 'face quail-selection-face))))
     ))
 
 (defun quail-display-buffer (buf)
@@ -965,7 +968,7 @@ The description about the current quail package is shown by \\[quail-help]."
 	(set-buffer quail-completion-buf)
 	(erase-buffer)
 	(setq quail-overlay (make-overlay 1 1))
-	(overlay-put quail-overlay 'face 'region)
+	(overlay-put quail-overlay 'face quail-selection-face)
 	(insert "Current candidates:\n")
 	(quail-completion-list key def 1)
 	(quail-display-buffer (current-buffer)))
