@@ -32,12 +32,14 @@
 ;;;		by Seo Young Jin <frog%math1.kaist.ac.kr@daiduk.kaist.ac.kr>
 ;;;	In hangle3 package, one entry is corrected.
 ;;; 93.7.27  modified for Mule Ver.0.9.8
-				by K.Un <zraun01@hpserv.zdv.uni-tuebingen.de>
+;;;				by K.Un <zraun01@hpserv.zdv.uni-tuebingen.de>
 ;;;	Several maps are corrected/added.
 ;;; 93.7.28  modified for Mule Ver.0.9.8 by H.Tsuiki <tsuiki@sfc.keio.ac.jp>
 ;;;	"o" produce $(C$@(B.
 ;;; 93.8.2   modified for Mule Ver.0.9.8 by K.Handa <handa@etl.go.jp>
 ;;;	hangle3 packaged is put in a separated file.
+;;; 93.11.19 modified for Mule Ver.1.1 by K.Handa <handa@etl.go.jp>
+;;;	"P" produces "$(C$F(B".
 
 (require 'quail)
 
@@ -47,20 +49,18 @@
    (?q . "$(C$2(B") (?t . "$(C$5(B") (?d . "$(C$7(B") (?w . "$(C$8(B") (?c . "$(C$:(B")
    (?z . "$(C$;(B") (?x . "$(C$<(B") (?v . "$(C$=(B") (?g . "$(C$>(B")
    (?R . "$(C$"(B") (?E . "$(C$((B") (?Q . "$(C$3(B") (?W . "$(C$9(B") (?T . "$(C$6(B")
-   (?O . "$(C@g(B")
+   (?O . "$(C@g(B") (?P . "$(C$F(B")
    (?k . "$(C$?(B") (?i . "$(C$A(B") (?j . "$(C$C(B") (?p . "$(C$D(B") (?u . "$(C$E(B") (?h . "$(C$G(B")
    (?y . "$(C$K(B") (?n . "$(C$L(B") (?b . "$(C$P(B") (?m . "$(C$Q(B") (?l . "$(C$S(B"))
  "Hangul input method.
-  $(CGQ1[(B:	$(C$!$"$$$'$($)$1$2$3$5$6$7$8$9$:$;$<$=$>$?$A$C$D$E$G$K$L$P$Q$S$@@g(B
-  KEY:	r R s e E f a q Q t T d w W c z x v g k i j p u h y n b m l o O
+  $(CGQ1[(B:	$(C$!$"$$$'$($)$1$2$3$5$6$7$8$9$:$;$<$=$>$?$A$C$D$F$E$G$K$L$P$Q$S$@@g(B
+  KEY:	r R s e E f a q Q t T d w W c z x v g k i j p P u h y n b m l o O
 --- Example ---
 <key sequence>		r	k	r	k
-<string inserted>	$(C$!(B	$(C0!(B	$(C0"(B	$(C0"$?(B
-<key sequence>		r	k	<SP>	r	k
-<string inserted>	$(C$!(B	$(C0!(B	$(C0!(B	$(C0!$!(B	$(C0!0!(B"
- *quail-mode-rich-map* t t)
-
-(defmacro qd (key tgt) (list 'quail-defrule key tgt))
+<string inserted>	$(C$!(B	$(C0!(B	$(C0"(B	$(C0!0!(B
+<key sequence>		r	k	r	<M-SP>	k
+<string inserted>	$(C$!(B	$(C0!(B	$(C0"(B	$(C0"(B	$(C0"$?(B"
+ nil t t)
 
 (qd "r"	?$(C$!(B)
 (qd "s"	?$(C$$(B)
@@ -82,6 +82,7 @@
 (qd "Q"	?$(C$3(B)
 (qd "W"	?$(C$9(B)
 (qd "T"	?$(C$6(B)
+(qd "P" ?$(C$F(B)
 
 (qd "k"	?$(C$?(B)
 (qd "i"	?$(C$A(B)
@@ -95,6 +96,7 @@
 (qd "m"	?$(C$Q(B)
 (qd "l"	?$(C$S(B)
 (qd "o"	?$(C$@(B)
+(qd "O"	?$(C@g(B)
 
 (qd "rk"	?$(C0!(B)
 (qd "rkr"	?$(C0"(B)
@@ -1672,7 +1674,6 @@
 (qd "wkT"	?$(C@d(B)
 (qd "wkd"	?$(C@e(B)
 (qd "wkw"	?$(C@f(B)
-(qd "O"	?$(C@g(B)
 (qd "Or"	?$(C@h(B)
 (qd "Os"	?$(C@i(B)
 (qd "Of"	?$(C@j(B)
@@ -2457,3 +2458,5 @@
 (qd "glq"	?$(CH|(B)
 (qd "glt"	?$(CH}(B)
 (qd "gld"	?$(CH~(B)
+
+(quail-setup-current-package)
