@@ -659,38 +659,47 @@ This is in addition to the primary selection.")
 
 ;; For MULE 94.5.25, 94.8.31
 
-(defconst x-default-fontset "default-fontset")
+(defvar x-default-fontset "default-fontset"
+  "Default fontset name.")
+
+(defvar x-default-font-alist
+  (list (cons lc-ascii "-*-fixed-medium-r-*--16-*-iso8859-1")
+	(cons lc-ltn1 "-*-fixed-medium-r-*--*-iso8859-1")
+	(cons lc-ltn2 "-*-fixed-medium-r-*--*-iso8859-2")
+	(cons lc-ltn3 "-*-fixed-medium-r-*--*-iso8859-3")
+	(cons lc-ltn4 "-*-fixed-medium-r-*--*-iso8859-4")
+	(cons lc-thai "-*-fixed-medium-r-*--*-tis620.2529-1")
+	(cons lc-grk "-*-fixed-medium-r-*--*-iso8859-7")
+	(cons lc-hbw "-*-fixed-medium-r-*--*-iso8859-8")
+	(cons lc-kana "-*-fixed-medium-r-*--*-jisx0201.1976-*")
+	(cons lc-roman "-*-fixed-medium-r-*--*-jisx0201.1976-*")
+	(cons lc-crl "-*-fixed-medium-r-*--*-iso8859-5")
+	(cons lc-ltn5 "-*-fixed-medium-r-*--*-iso8859-9")
+	(cons lc-jpold "-*-fixed-medium-r-*--*-jisx0208.1983-*")
+	(cons lc-cn "-*-*-medium-r-*--*-gb2312.1980-*")
+	(cons lc-jp "-*-fixed-medium-r-*--*-jisx0208.1983-*")
+	(cons lc-kr "-*-mincho-medium-r-*--*-ksc5601.1987-*")
+	(cons lc-jp2 "-*-fixed-medium-r-*--*-jisx0212*-*")
+	(cons lc-cns1 "-*-fixed-medium-r-*--*-cns11643*-0")
+	(cons lc-cns2 "-*-fixed-medium-r-*--*-cns11643*-1")
+	(cons lc-big5-1 "-*-fixed-medium-r-*--*-big5*-*")
+	(cons lc-big5-2 "-*-fixed-medium-r-*--*-big5*-*")
+	(cons lc-sisheng "-*-fixed-medium-r-*--*-sisheng_cwnn-0")
+	(cons lc-vn-1 "-*-fixed-medium-r-*--*-viscii1.1-1")
+	(cons lc-vn-2 "-*-fixed-medium-r-*--*-viscii1.1-1")
+	(cons lc-arb0 "-*-fixed-medium-r-*--*-mulearabic-0")
+	(cons lc-arb1 "-*-fixed-medium-r-*--*-mulearabic-1")
+	(cons lc-arb2 "-*-fixed-medium-r-*--*-mulearabic-2")
+	(cons lc-ipa0 "-*-fixed-medium-r-*--*-muleipa-1")
+	(cons lc-ethio "-*-fixed-medium-r-*--*-ethio-0")
+	)
+  "Alist saying which font to be used by default for each character set.
+Each element looks like (LEADING-CHAR FONTNAME);
+LEADING-CHAR is an integer identifying character set (e.g. lc-ascii, lc-jp),
+FONTNAME is a string of fontname to be used in each window system.")
+
 (defconst x-default-fonts (make-vector 128 nil))
-(let ((fonts (list (cons lc-ascii "-*-fixed-medium-r-*--16-*-iso8859-1")
-		   (cons lc-ltn1 "-*-fixed-medium-r-*--*-iso8859-1")
-		   (cons lc-ltn2 "-*-fixed-medium-r-*--*-iso8859-2")
-		   (cons lc-ltn3 "-*-fixed-medium-r-*--*-iso8859-3")
-		   (cons lc-ltn4 "-*-fixed-medium-r-*--*-iso8859-4")
-		   (cons lc-thai "-*-fixed-medium-r-*--*-tis620.2529-1")
-		   (cons lc-grk "-*-fixed-medium-r-*--*-iso8859-7")
-		   (cons lc-hbw "-*-fixed-medium-r-*--*-iso8859-8")
-		   (cons lc-kana "-*-fixed-medium-r-*--*-jisx0201.1976-*")
-		   (cons lc-roman "-*-fixed-medium-r-*--*-jisx0201.1976-*")
-		   (cons lc-crl "-*-fixed-medium-r-*--*-iso8859-5")
-		   (cons lc-ltn5 "-*-fixed-medium-r-*--*-iso8859-9")
-		   (cons lc-jpold "-*-fixed-medium-r-*--*-jisx0208.1983-*")
-		   (cons lc-cn "-*-*-medium-r-*--*-gb2312.1980-*")
-		   (cons lc-jp "-*-fixed-medium-r-*--*-jisx0208.1983-*")
-		   (cons lc-kr "-*-mincho-medium-r-*--*-ksc5601.1987-*")
-		   (cons lc-jp2 "-*-fixed-medium-r-*--*-jisx0212*-*")
-		   (cons lc-cns1 "-*-fixed-medium-r-*--*-cns11643*-0")
-		   (cons lc-cns2 "-*-fixed-medium-r-*--*-cns11643*-1")
-		   (cons lc-big5-1 "-*-fixed-medium-r-*--*-big5*-*")
-		   (cons lc-big5-2 "-*-fixed-medium-r-*--*-big5*-*")
-		   (cons lc-sisheng "-*-fixed-medium-r-*--*-sisheng_cwnn-0")
-		   (cons lc-vn-1 "-*-fixed-medium-r-*--*-viscii1.1-1")
-		   (cons lc-vn-2 "-*-fixed-medium-r-*--*-viscii1.1-1")
-		   (cons lc-arb0 "-*-fixed-medium-r-*--*-mulearabic-0")
-		   (cons lc-arb1 "-*-fixed-medium-r-*--*-mulearabic-1")
-		   (cons lc-arb2 "-*-fixed-medium-r-*--*-mulearabic-2")
-		   (cons lc-ipa0 "-*-fixed-medium-r-*--*-muleipa-1")
-		   (cons lc-ethio "-*-fixed-medium-r-*--*-ethio-0")
-		   )))
+(let ((fonts x-default-font-alist))
   (while fonts
     (aset x-default-fonts (logand (car (car fonts)) 127) (cdr (car fonts)))
     (setq fonts (cdr fonts))))
