@@ -451,6 +451,15 @@ extern Lisp_Object Vtransient_mark_mode;
 #define BUFFER_REALLOC(data,size) ((unsigned char *) r_re_alloc (&data, (size)))
 #define BUFFER_FREE(data) (r_alloc_free (&data))
 #define R_ALLOC_DECLARE(var,data) (r_alloc_declare (&var, (data)))
+#ifdef OSF1
+#ifdef __STDC__
+extern void *r_alloc();
+extern void *r_re_alloc();
+#else
+extern char *r_alloc();
+extern char *r_re_alloc();
+#endif
+#endif
 #else
 #define BUFFER_ALLOC(data,size) (data = (unsigned char *) malloc ((size)))
 #define BUFFER_REALLOC(data,size) ((unsigned char *) realloc ((data), (size)))
