@@ -699,6 +699,7 @@ DEFUN ("canna-henkan-begin", Fcanna_henkan_begin, Scanna_henkan_begin,
        1, 1, 0,
 "ʴѴִ̤Ԥ롣ʸڤ꤬Ƥ롣")
 	(yomi)
+    Lisp_Object yomi;
 {
   int nbun;
   Lisp_Object res;
@@ -751,6 +752,7 @@ DEFUN ("canna-henkan-next", Fcanna_henkan_next, Scanna_henkan_next,
        1, 1, 0,
 "롣")
 	(bunsetsu)
+    Lisp_Object bunsetsu;
 {
   int i, nbun, slen, len;
   unsigned char *p, RkBuf[RKBUFSIZE];
@@ -780,6 +782,7 @@ DEFUN ("canna-bunsetu-henkou", Fcanna_bunsetu_henkou, Scanna_bunsetu_henkou,
        2, 2, 0,
 "ʸĹꤹ롣")
 	(bunsetsu, bunlen)
+    Lisp_Object bunsetsu, bunlen;
 {
   int nbun, len;
 
