@@ -582,6 +582,31 @@ You can also provide a character as an argument.")
 }
 /* end of patch */
 
+DEFUN ("set-char-registry", Fset_char_registry, Sset_char_registry, 2, 2, 0,
+  "Set registry name of CHARSET to REGISTRY.\n\
+CHARSET should be specified by a leading character.\n\
+You can also provide a character as an argument.\n\
+Return REGISTRY, or nil if CHARSET is invalid.")
+  (charset, registry)
+     Lisp_Object charset;
+{
+  unsigned int c;
+  char *r;
+
+  CHECK_NUMBER (charset, 0);
+  c = XFASTINT (charset);
+  if (MC_CHAR_P (c))
+    c = CHAR_LEADING_CHAR (c);
+  if (VALID_LC (c) && char_registry[c & 0x7F]) {
+    r = (char *)xmalloc (XSTRING (registry)->size + 1);
+    bcopy (XSTRING (registry)->data, r, XSTRING (registry)->size + 1);
+    char_registry[c & 0x7F] = r;
+    return (registry);
+  }
+  else
+    return Qnil;
+}
+
 DEFUN ("char-description", Fchar_description, Schar_description, 1, 1, 0,
   "Return description of CHARSET.\n\
 CHARSET should be specified by a leading character.\n\
@@ -705,10 +730,10 @@ search_cmpchar(str, len)
   if (hashp->size == 0) {
     hashp->size = 8;
     hashp->used = 0;
-    hashp->table = xmalloc (sizeof (hashp->table[0]) * hashp->size);
+    hashp->table = (int *) xmalloc (sizeof (hashp->table[0]) * hashp->size);
   } else if (hashp->size <= hashp->used) {
     hashp->size *= 2;
-    hashp->table = xrealloc (hashp->table,
+    hashp->table = (int *) xrealloc (hashp->table,
 			     sizeof (hashp->table[0]) * hashp->size);
   }
   hashp->table[hashp->used++] = n_cmpchars;
@@ -726,7 +751,7 @@ search_cmpchar(str, len)
   cmpchar_table[n_cmpchars].data = buf;
 
   {
-    GLYPH *g = (GLYPH *) alloca (sizeof (*g) * 16);
+    GLYPH g[16];
     unsigned char *p, *q;
     int i, width = 1;
 
@@ -744,8 +769,8 @@ search_cmpchar(str, len)
       }
     }
     cmpchar_table[n_cmpchars].glyph_len = i;
-    cmpchar_table[n_cmpchars].glyph = (GLYPH *) xmalloc (sizeof (*g) * i);
-    bcopy (g, cmpchar_table[n_cmpchars].glyph, sizeof (*g) * i);
+    cmpchar_table[n_cmpchars].glyph = (GLYPH *) xmalloc ((sizeof g[0]) * i);
+    bcopy (g, cmpchar_table[n_cmpchars].glyph, (sizeof g[0]) * i);
     cmpchar_table[n_cmpchars].width = width;
   }
 
@@ -815,6 +840,7 @@ syms_of_charset ()
   defsubr (&Schar_bytes);
   defsubr (&Schar_direction);  /* 92.8.2 by Y.Niibe */
   defsubr (&Schar_registry);	/* 94.6.3 by K.Handa */
+  defsubr (&Sset_char_registry);
   defsubr (&Schar_description);
   defsubr (&Schars_in_string);
 
