@@ -1175,8 +1175,9 @@ direct_output_for_insert (g)
 }
 
 int
-direct_output_forward_char (n)
+direct_output_forward_char (n, right)
      int n;
+     int right;
 {
   register FRAME_PTR frame = selected_frame;
   register struct window *w = XWINDOW (selected_window);
@@ -1219,6 +1220,21 @@ direct_output_forward_char (n)
     return 0;
 #endif
 
+  /* If the direction of character has changed, must re calculate
+     cursor position.  The display direction of the left hand side
+     character is always LEFT_TO_RIGHT, becase it is 0x20 <= c < 0x7e.  */
+  if (!NILP (current_buffer->mc_flag) && right < ZV)
+    {
+      int c = FETCH_CHAR (right); /* right hand char of after cursor motion */
+
+      if (c == LCCMP) /* FIXME */
+	return 0;
+
+      if ((LC_P (c) && char_direction[c] != LEFT_TO_RIGHT)
+	  || (c == '\n' && !NILP (current_buffer->display_direction)))
+	return 0;
+    }
+
   FRAME_CURSOR_X (frame) += n;
   XFASTINT (w->last_point_x) = FRAME_CURSOR_X (frame);
   XFASTINT (w->last_point) = point;
