@@ -249,20 +249,29 @@ int mc_readlink (path, buf, size)
   unsigned char buffer[MC_MAXPATHLEN], buffer2[MAXPATHLEN];
   int nread;
 
+  /* decode given `path' into buffer, and get the link contents into
+     buffer2. */
   nread = readlink (decode_path (path, buffer), buffer2, MAXPATHLEN);
   if (nread > 0)
     {
       int len;
-      unsigned char *p;
+      unsigned char *result;
 
       len = encode_path_1 (buffer2, nread, buffer, sizeof (buffer));
+
+      /* if the conversion success and the result fits to `buf', use
+         it. */
       if (0 <= len && len <= size)
 	{
-	  bcopy (buffer, buf, len);
-	  return len;
+	  nread = len;
+	  result = buffer;
 	}
+      else			/* use original one. */
+	result = buffer2;
+	
+      bcopy (result, buf, nread);
     }
-  return -1;
+  return nread;
 }
 #endif
 
