@@ -236,6 +236,7 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
   Fmake_local_variable (Qprint_escape_newlines);
   print_escape_newlines = 1;
 
+  minibuf_level++;
   record_unwind_protect (read_minibuf_unwind, Qnil);
 
   Vminibuf_scroll_window = selected_window;
@@ -244,7 +245,6 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
   XFASTINT (XWINDOW (minibuf_window)->hscroll) = 0;
 
   Ferase_buffer ();
-  minibuf_level++;
 
   if (!NILP (initial))
     {
@@ -1604,7 +1604,7 @@ DEFUN ("minibuffer-extend-prompt", Fminibuffer_extend_prompt,
 
   CHECK_STRING (str, 0);
   minibuf_prompt
-    = (char *) xrealloc (minibuf_prompt,
+    = (Lisp_Object) xrealloc (minibuf_prompt,
 			 minibuf_prompt_width + XSTRING (str)->size + 1);
   bcopy (XSTRING (str)->data, minibuf_prompt + minibuf_prompt_width,
 	 XSTRING (str)->size + 1);
