@@ -2,7 +2,7 @@
         Coded by Yutaka Ishikawa at ETL (yisikawa@etl.go.jp)
                  Satoru Tomura   at ETL (tomura@etl.go.jp)
 	Modified for Wnn4 library by
-		 Toshiaki Shingu of Canon Inc. (shingu@cpr.canon.co.jp)
+		 Toshiaki Shingu (shingu@cpr.canon.co.jp)
 		 Hiroshi Kuribayashi (kuri@nff.ncl.omron.co.jp)
 
    This file is part of Egg on Nemacs (Japanese environment)
@@ -26,6 +26,8 @@
 /*
  *      CHANGE LOG
  *
+ *	1994.2.24  check_wnn_server_type() should not be static.
+ *	1994.2.3   kWnn support by H.Kuribayashi.
  *	1993.6.29  wnn-server-set-rev updated.
  *	1993.4.23  serious bug in wnn-server-open fixed.
  *	1993.4.2   wnn-server-fuzokugo-set,
@@ -90,8 +92,8 @@
  *		$B:o=|$9$k!#(B
  *
  *	   (wnn-server-dict-list)
- *		RETURNS: ((dic-no1 file-name1 comment1 word-no1)
- *			  (dic-no2 file-name2 comment2 word-no2)...)
+ *		RETURNS: ((dic-no1 file-name1 comment1 word-no1 nice1)
+ *			  (dic-no2 file-name2 comment2 word-no2 nice2)...)
  *		DESCRIPTION: $B%P%C%U%!>e$N<-=q$N%j%9%H$rF@$k!#(B
  *
  *	   (wnn-server-dict-comment dic-no comment)
@@ -293,28 +295,32 @@
 #include "jllib.h"
 #include "cplib.h"
 
+/* UCHAR $B$,Fs=EDj5A$5$l$k$N$G(B */
+#define _UCHAR_T
+
 #define EGG_TIMEOUT 5
-#define NSERVER 2
+#define NSERVER 4
 #define WNNSERVER_J 0
 #define WNNSERVER_C 1
+#define WNNSERVER_T 2
+#define WNNSERVER_K 3
 
-#ifdef NULL
-#undef NULL
-#endif NULL
 #include "lisp.h"
 #include "buffer.h"
 #include "window.h"
-#include "mule.h"
+#include "charset.h"
 
 static struct wnn_buf *wnnfns_buf[NSERVER];
 static struct wnn_env *wnnfns_env_norm[NSERVER];
 static struct wnn_env *wnnfns_env_rev[NSERVER];
 static int wnnfns_norm;
+static unsigned char lc_wnn_server_type[NSERVER] = {LCJP, LCCN, LCINV, LCKR};
 
 /* Lisp Variables and Constants Definition */
 Lisp_Object	Qjserver;
 Lisp_Object	Qcserver;
 /*Lisp_Object	Qtserver;*/
+Lisp_Object	Qkserver;
 Lisp_Object	Qwnn_no_uniq;
 Lisp_Object	Qwnn_uniq;
 Lisp_Object	Qwnn_uniq_kanji;
@@ -342,13 +348,22 @@ Return nil if error occurs")
   int	snum;
   CHECK_STRING (lname, 0);
 
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
+  snum = check_wnn_server_type();
+  switch (snum) {
+  case WNNSERVER_J:
     strcpy(langname, "ja_JP");
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
+    break;
+  case WNNSERVER_C:
     strcpy(langname, "zh_CN");
+    break;
+/*  
+  case WNNSERVER_T:
+    strcpy(langname, "zh_TW");
+    break;
+*/
+  case WNNSERVER_K:
+    strcpy(langname, "ko_KR");
+    break;
   }
   strncpy(envname, XSTRING(lname)->data, 32);
   if (hname == Qnil) strcpy(hostname, "");
@@ -382,12 +397,7 @@ are not saved.")
      ()
 {
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(wnnfns_env_norm[snum]) {
     if (Vwnnenv_sticky == Qnil) jl_env_un_sticky_e(wnnfns_env_norm[snum]);
@@ -423,12 +433,7 @@ Specify password files of dictionary and frequency, PW1 and PW2, if needed.")
   CHECK_NUMBER (args[2], 2);
   if (args[5] != Qnil) CHECK_STRING (args[5], 5);
   if (args[6] != Qnil) CHECK_STRING (args[6], 6);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   GCPRO1 (*args);
   gcpro1.nvars = nargs;
@@ -459,12 +464,7 @@ DEFUN ("wnn-server-dict-delete", Fwnn_dict_delete, Swnn_dict_delete,
   int	no;
   int	snum;
   CHECK_NUMBER(dicno, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   no = XINT(dicno);
   if(!wnnfns_buf[snum]) return Qnil;
   if(jl_dic_delete(wnnfns_buf[snum], no) < 0) return Qnil;
@@ -483,14 +483,8 @@ DEFUN ("wnn-server-dict-list", Fwnn_dict_list, Swnn_dict_list,
   int	snum;  
   unsigned char lc;
 
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   if((cnt = jl_dic_list(wnnfns_buf[snum], &dicinfo)) < 0) return Qnil;
   val = Qnil;
@@ -500,7 +494,8 @@ DEFUN ("wnn-server-dict-list", Fwnn_dict_list, Swnn_dict_list,
     val = Fcons(Fcons(make_number(dicinfo->dic_no),
 		Fcons(make_string(dicinfo->fname, strlen(dicinfo->fname)),
 		Fcons(make_string(comment, strlen(comment)),
-		Fcons(make_number(dicinfo->gosuu), Qnil)))), val);
+		Fcons(make_number(dicinfo->gosuu),
+		Fcons(make_number(dicinfo->nice), Qnil))))), val);
 	}
   return val;
 }
@@ -516,12 +511,7 @@ Comment string COMMENT")
   int	snum;
   CHECK_NUMBER(dicno, 0);
   CHECK_STRING(comment, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   m2w(XSTRING(comment)->data, wbuf);
   if(jl_dic_comment_set(wnnfns_buf[snum], XINT(dicno), wbuf) < 0) 
@@ -537,12 +527,7 @@ DEFUN ("wnn-server-set-rev", Fwnn_set_rev, Swnn_set_rev,
      register Lisp_Object rev;
 {
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(rev == Qnil){
     if((!wnnfns_buf[snum]) || (!wnnfns_env_norm[snum])) return;
     jl_env_set(wnnfns_buf[snum], wnnfns_env_norm[snum]);
@@ -565,15 +550,9 @@ DEFUN ("wnn-server-henkan-begin", Fwnn_begin_henkan, Swnn_begin_henkan,
   w_char		wbuf[5000];
   int	snum;
   CHECK_STRING(hstring, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   m2w(XSTRING(hstring)->data, wbuf);
-  /* if (wnnfns_lang == LCCN) */
   if (snum == WNNSERVER_C)	/* 92.10.30 by T.Saneto */
       w2y(wbuf);
 
@@ -592,12 +571,7 @@ if dai-bunsetsu, NIL if sho-bunsetsu. Return the current offset of zenkouho.")
   int	snum;
   int	uniq_level;
   CHECK_NUMBER(bunNo, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   no = XINT(bunNo);
   if (Vwnn_uniq_level == Qwnn_no_uniq) uniq_level = WNN_NO_UNIQ;
@@ -626,14 +600,8 @@ DEFUN ("wnn-server-get-zenkouho", Fwnn_get_zenkouho, Swnn_get_zenkouho,
   int	snum;
   unsigned char lc;
   CHECK_NUMBER(kouhoNo, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   jl_get_zenkouho_kanji(wnnfns_buf[snum], XINT(kouhoNo), wbuf);
   w2m(wbuf, kanji_buf, lc);
@@ -646,12 +614,7 @@ DEFUN ("wnn-server-zenkouho-bun", Fwnn_zenkouho_bun, Swnn_zenkouho_bun,
      ()
 {
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   return make_number(jl_zenkouho_bun(wnnfns_buf[snum]));
 }
 
@@ -661,12 +624,7 @@ DEFUN ("wnn-server-zenkouho-suu", Fwnn_zenkouho_suu, Swnn_zenkouho_suu,
      ()
 {
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   return make_number(jl_zenkouho_suu(wnnfns_buf[snum]));
 }
 
@@ -677,12 +635,7 @@ DEFUN ("wnn-server-dai-top", Fwnn_dai_top, Swnn_dai_top, 1, 1, 0,
 {
   int	snum;
   CHECK_NUMBER(bunNo, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if (jl_dai_top(wnnfns_buf[snum], XINT(bunNo)) == 1) return Qt;
   else return Qnil;
@@ -695,12 +648,7 @@ DEFUN ("wnn-server-dai-end", Fwnn_dai_end, Swnn_dai_end, 1, 1, 0,
 {
   int	snum;
   CHECK_NUMBER(bunNo, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   return make_number(dai_end(XINT(bunNo), snum));
 }
@@ -712,12 +660,7 @@ DEFUN ("wnn-server-henkan-kakutei", Fwnn_kakutei, Swnn_kakutei, 2, 2, 0,
 {
   int	snum;
   CHECK_NUMBER(offset, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(dai == Qnil) {
     if(jl_set_jikouho(wnnfns_buf[snum], XINT(offset)) < 0) return Qnil;
@@ -739,12 +682,7 @@ DEFUN ("wnn-server-bunsetu-henkou", Fwnn_bunsetu_henkou, Swnn_bunsetu_henkou,
   int	snum;
   CHECK_NUMBER(bunNo, 0);
   CHECK_NUMBER(len, 1);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   no = XINT(bunNo);
   if((cnt = jl_nobi_conv(wnnfns_buf[snum], no, XINT(len), -1, WNN_USE_MAE,
@@ -766,14 +704,8 @@ DEFUN ("wnn-server-inspect", Fwnn_inspect, Swnn_inspect, 1, 1, 0,
   int	snum;
   unsigned char		lc;
   CHECK_NUMBER(bunNo, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   bun_no = XINT(bunNo);
   val = Qnil;
@@ -803,12 +735,7 @@ DEFUN ("wnn-server-henkan-quit", Fwnn_quit_henkan, Swnn_quit_henkan, 0, 0, 0,
      ()
 {
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   return Qt;
 }
@@ -826,14 +753,8 @@ DEFUN ("wnn-server-bunsetu-kanji", Fwnn_bunsetu_kanji, Swnn_bunsetu_kanji,
   int			snum;
   unsigned char		lc;
   CHECK_NUMBER(bunNo, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   no = XINT(bunNo);
   kanji_len = jl_get_kanji(wnnfns_buf[snum], no, no + 1, wbuf);
@@ -855,14 +776,8 @@ DEFUN ("wnn-server-bunsetu-yomi", Fwnn_bunsetu_yomi, Swnn_bunsetu_yomi,
   int			snum;
   unsigned char		lc;
   CHECK_NUMBER(bunNo, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   no = XINT(bunNo);
   yomi_len = jl_get_yomi(wnnfns_buf[snum], no, no + 1, wbuf);
@@ -877,12 +792,7 @@ DEFUN ("wnn-server-bunsetu-suu", Fwnn_bunsetu_suu, Swnn_bunsetu_suu,
      ()
 {
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   return make_number(jl_bun_suu(wnnfns_buf[snum]));
 }
@@ -896,12 +806,7 @@ DEFUN ("wnn-server-hindo-update", Fwnn_hindo_update, Swnn_hindo_update,
   int		no;
   Lisp_Object	val;
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if (bunNo == Qnil) no = -1;
   else {
     CHECK_NUMBER(bunNo, 0);
@@ -917,7 +822,7 @@ DEFUN ("wnn-server-word-add", Fwnn_word_toroku, Swnn_word_toroku, 5, 5, 0,
        "Add a word to dictionary. Arguments are\n\
 DIC-NUMBER, KANJI, YOMI, COMMENT, HINSI-NUMBER")
      (dicno, kanji, yomi, comment, hinsi)
-     register Lisp_Object dicno, kanji, yomi, hinsi;
+     register Lisp_Object dicno, kanji, yomi, comment, hinsi;
 {
   w_char		yomi_buf[256], kanji_buf[256], comment_buf[256];
   int	snum;
@@ -926,12 +831,7 @@ DIC-NUMBER, KANJI, YOMI, COMMENT, HINSI-NUMBER")
   CHECK_STRING(yomi, 2);
   CHECK_STRING(comment, 3);
   CHECK_NUMBER(hinsi, 4);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   m2w(XSTRING(yomi)->data, yomi_buf);
   if (snum == WNNSERVER_C)
@@ -953,12 +853,7 @@ DEFUN ("wnn-server-word-delete", Fwnn_word_sakujo, Swnn_word_sakujo, 2, 2, 0,
   int	snum;
   CHECK_NUMBER(no, 0);
   CHECK_NUMBER(serial, 1);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(jl_word_delete(wnnfns_buf[snum], XINT(no), XINT(serial)) < 0)
     return Qnil;
@@ -974,12 +869,7 @@ DEFUN ("wnn-server-word-use", Fwnn_word_use, Swnn_word_use, 2, 2, 0,
   int	snum;
   CHECK_NUMBER(no, 0);
   CHECK_NUMBER(serial, 1);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(jl_word_use(wnnfns_buf[snum], XINT(no), XINT(serial)) < 0)
     return Qnil;
@@ -998,14 +888,8 @@ DEFUN ("wnn-server-word-info", Fwnn_word_info, Swnn_word_info, 2, 2, 0,
   unsigned char		lc;
   CHECK_NUMBER(no, 0);
   CHECK_NUMBER(serial, 1);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   /* 92.10.19 patch by T.Atsushiba <atsushiba@ailove.ENET.dec.com>
      -- coerced to (int) */
@@ -1036,12 +920,7 @@ SERIAL-NUMBER, FREQUENCY")
   CHECK_NUMBER(no, 0);
   CHECK_NUMBER(serial, 1);
   CHECK_NUMBER(hindo, 2);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(js_hindo_set(jl_env_get(wnnfns_buf[snum]),
 		  XINT(no),
@@ -1068,14 +947,8 @@ Return list of (kanji hinshi freq dic_no serial).")
   int			snum;
   unsigned char		lc;
   CHECK_STRING(yomi, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   m2w(XSTRING(yomi)->data, wbuf);
   if (snum == WNNSERVER_C)
@@ -1102,12 +975,7 @@ DEFUN ("wnn-server-dict-save", Fwnn_dict_save, Swnn_dict_save, 0, 0, 0,
      ()
 {
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(jl_dic_save_all(wnnfns_buf[snum]) < 0) return Qnil;
   else return Qt;
@@ -1121,12 +989,7 @@ suuji kana eisuu kigou toji_kakko fuzokogo kaikakko)")
 {
   struct wnn_param	param;
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(jl_param_get(wnnfns_buf[snum], &param) < 0) return Qnil;
   return Fcons(make_number(param.n),
@@ -1160,12 +1023,7 @@ suuji kana eisuu kigou toji_kakko fuzokogo kaikakko")
   struct wnn_param	param;
   int	snum;
   for (rc = 0; rc < 17; rc++) CHECK_NUMBER(args[rc], rc);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   param.n = XINT(args[0]);
   param.nsho = XINT(args[1]);
@@ -1200,15 +1058,23 @@ DEFUN ("wnn-server-get-msg", Fwnn_get_msg, Swnn_get_msg, 0, 0, 0,
   unsigned char		lc;
   char  langname[32];
 /*  CHECK_NUMBER(errno, 0);*/
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
+  switch (snum) {
+  case WNNSERVER_J:
     strcpy(langname, "ja_JP");
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
+    break;
+  case WNNSERVER_C:
     strcpy(langname, "zh_CN");
+    break;
+/*
+  case WNNSERVER_T:
+    strcpy(langname, "zh_TW");
+    break;
+*/
+  case WNNSERVER_K:
+    strcpy(langname, "ko_KR");
+    break;
   }
   if(!wnnfns_buf[snum]) return Qnil;
 /*  msgp = msg_get(wnn_msg_cat, XINT(errno), 0, 0);*/
@@ -1225,12 +1091,7 @@ DEFUN ("wnn-server-fuzokugo-set", Fwnn_fuzokugo_set, Swnn_fuzokugo_set,
 {
   int	snum;
   CHECK_STRING(file, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(jl_fuzokugo_set(wnnfns_buf[snum], XSTRING(file)->data) < 0)
     return Qnil;
@@ -1243,12 +1104,7 @@ DEFUN ("wnn-server-fuzokugo-get", Fwnn_fuzokugo_get, Swnn_fuzokugo_get,
 {
   char	fname[256];
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(jl_fuzokugo_get(wnnfns_buf[snum], fname) < 0) return Qnil;
   return make_string(fname, strlen(fname));
@@ -1260,12 +1116,7 @@ DEFUN ("wnn-server-isconnect", Fwnn_isconnect, Swnn_isconnect, 0, 0, 0,
      ()
 {
   int	snum;
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if(jl_isconnect(wnnfns_buf[snum])) return Qt;
   else return Qnil;
@@ -1281,12 +1132,7 @@ DEFUN ("wnn-server-hinsi-dicts", Fwnn_hinsi_dicts, Swnn_hinsi_dicts, 1, 1, 0,
   Lisp_Object	val;
   int	snum;
   CHECK_NUMBER(hinsi, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   if((cnt = jl_hinsi_dicts(wnnfns_buf[snum], XINT(hinsi), &area)) < 0)
     return Qnil;
@@ -1312,14 +1158,8 @@ DEFUN ("wnn-server-hinsi-list", Fwnn_hinsi_list, Swnn_hinsi_list, 2, 2, 0,
   unsigned char lc;
   CHECK_NUMBER(dicno, 0);
   CHECK_STRING(name, 1);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   m2w(XSTRING(name)->data, wbuf);
   if((cnt = jl_hinsi_list(wnnfns_buf[snum], XINT(dicno), wbuf, &area)) < 0)
@@ -1344,14 +1184,8 @@ DEFUN ("wnn-server-hinsi-name", Fwnn_hinsi_name, Swnn_hinsi_name, 1, 1, 0,
   int			snum;
   unsigned char		lc;
   CHECK_NUMBER(no, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-    lc = LCJP;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-    lc = LCCN;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
+  lc = lc_wnn_server_type[snum];
   if(!wnnfns_buf[snum]) return Qnil;
   if((wname = jl_hinsi_name(wnnfns_buf[snum], XINT(no))) == 0) return Qnil;
   w2m(wname, name, lc);
@@ -1368,43 +1202,13 @@ DEFUN ("wnn-server-hinsi-number", Fwnn_hinsi_number, Swnn_hinsi_number,
   int		no;
   int	snum;
   CHECK_STRING(name, 0);
-  if (Vwnn_server_type == Qjserver) {
-    snum = WNNSERVER_J;
-  }
-  else if (Vwnn_server_type == Qcserver) {
-    snum = WNNSERVER_C;
-  }
+  if ((snum = check_wnn_server_type()) == -1) return Qnil;
   if(!wnnfns_buf[snum]) return Qnil;
   m2w(XSTRING(name)->data, w_buf);
   if((no = jl_hinsi_number(wnnfns_buf[snum], w_buf)) < 0) return Qnil;
   return make_number(no);
 }
 
-wnn_exit()
-{
-  int	i;
-  for (i = 0; i < NSERVER; i++) {
-    if(!wnnfns_buf[i]) continue;
-    if(!jl_isconnect(wnnfns_buf[i])) continue;
-    if(wnnfns_env_norm[i]) {
-      jl_dic_save_all_e(wnnfns_env_norm[i]);
-      if (Vwnnenv_sticky == Qnil) jl_env_un_sticky_e(wnnfns_env_norm[i]);
-      else jl_env_sticky_e(wnnfns_env_norm[i]);
-      jl_disconnect(wnnfns_env_norm[i]);
-    }
-    if(wnnfns_env_rev[i]) {
-      jl_dic_save_all_e(wnnfns_env_rev[i]);
-      if (Vwnnenv_sticky == Qnil) jl_env_un_sticky_e(wnnfns_env_rev[i]);
-      else jl_env_sticky_e(wnnfns_env_rev[i]);
-      jl_disconnect(wnnfns_env_rev[i]);
-    }
-    jl_env_set(wnnfns_buf[i], 0);
-    jl_close(wnnfns_buf[i]);
-    wnnfns_buf[i] = (struct wnn_buf *)0;
-    wnnfns_env_norm[i] = wnnfns_env_rev[i] = (struct wnn_env *)0;
-  }
-}
-
 syms_of_wnn()
 {
   int i;
@@ -1458,6 +1262,8 @@ syms_of_wnn()
 
   Qjserver = intern("jserver");
   Qcserver = intern("cserver");
+  /* Qtserver = intern("tserver"); */
+  Qkserver = intern("kserver");
 
   Qwnn_no_uniq = intern("wnn-no-uniq");
   Qwnn_uniq = intern("wnn-uniq");
@@ -1543,6 +1349,7 @@ m2w(mp, wp)
       case LCKANA: *wp++ = *mp++; break;
       case LCROMAN: *wp++ = *mp++ & 0x7F; break;
       case LCJPOLD: case LCCN: case LCJP: case LCKR:
+      /* case LCTW: */
 	ch = *mp++;
 	*wp++ = (ch << 8) | *mp++;
 	break;
@@ -1633,14 +1440,18 @@ unsigned char *s;
   extern Lisp_Object	Fy_or_n_p();
   unsigned char		mbuf[512];
   unsigned char		lc;
-  if (Vwnn_server_type == Qjserver) lc = LCJP;
-  else if (Vwnn_server_type == Qcserver) lc = LCCN;
+  int			len;
+  int			snum;
+  if ((snum  = check_wnn_server_type()) == -1) return 0;
+  lc = lc_wnn_server_type[snum];
   /* if no message found, create file without query */
 /*  if (wnn_msg_cat->msg_bd == 0) return 1;*/
   if (*s == 0) return 1;
   c2m(s, mbuf, lc);
-  /* truncate "(Y/N)". ad-hoc coding */
-  if (Fy_or_n_p(make_string(mbuf, strlen(mbuf) - 5)) == Qnil) return 0;
+  /* truncate "(Y/N)" */
+  for (len = 0; (mbuf[len]) && (len < 512); len++);
+  for (; (mbuf[len] != '(') && (len > 0); len--);
+  if (Fy_or_n_p(make_string(mbuf, len)) == Qnil) return 0;
   else return (1);
 }
 
@@ -1652,9 +1463,28 @@ char *s;
   char			mbuf[512];
   unsigned char		lc;
   extern Lisp_Object 	Fmessage();
-  if (Vwnn_server_type == Qjserver) lc = LCJP;
-  else if (Vwnn_server_type == Qcserver) lc = LCCN;
+  int			snum;
+  if ((snum = check_wnn_server_type()) == -1) return;
+  lc = lc_wnn_server_type[snum];
   c2m(s, mbuf, lc);
   args[0] = make_string(mbuf, strlen(mbuf));
   Fmessage(1, args);
 }
+
+int
+check_wnn_server_type()
+{
+  if (Vwnn_server_type == Qjserver) {
+    return WNNSERVER_J;
+  }
+  else if (Vwnn_server_type == Qcserver) {
+    return WNNSERVER_C;
+  }
+  /* else if (Vwnn_server_type == Qtserver) {
+    return WNNSERVER_T;
+  } */
+  else if (Vwnn_server_type == Qkserver) {
+    return WNNSERVER_K;
+  }
+  else return -1;
+}
